package pps.core.task.job;

import org.springframework.util.CollectionUtils;
import pps.cloud.framedata.service.CommonDataService;
import pps.cloud.framedata.service.data.common_data.GetColumnListInput;
import pps.cloud.framedata.service.data.common_data.GetColumnListOutput;
import pps.cloud.framedata.service.data.common_data.SelectCommonInput;
import pps.cloud.framedata.service.data.common_data.SelectCommonOutput;
import pps.cloud.system.service.SysBaseInfoSyncService;
import pps.cloud.system.service.data.GetSysBaseInfoSysncConfigOutput;
import pps.core.common.utils.NamedServiceProvider;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XDateUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.helper.XLockHelper;
import xstartup.helper.XTransactionHelper;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author lixueyan
 * @date 2023/2/28 0028 14:47
 */

@XText("基础数据同步到多库的任务")
@XService
public class SyncBaseInfoJob implements XJob {

    @XCronTrigger(value = "0 0/5 * * * ?")
    @Override
    public XServiceResult execute(XContext context) {
        SysBaseInfoSyncService service = context.getBean(SysBaseInfoSyncService.class);
        //获取同步配置
        XListResult<GetSysBaseInfoSysncConfigOutput> listResult = service.querySyncConfig(context);
        listResult.throwIfFail();
        List<GetSysBaseInfoSysncConfigOutput> entList = listResult.getResult();
        //执行同步操作
        XServiceResult lockResult = XLockHelper.lock(context, "sync_base_info", "sync_base_info", 60, () -> {

        });
        lockResult.throwIfFail(); //为防止规定时间内 没有处理完成， 对业务枷锁
        Date nowDate = new Date();
        for (GetSysBaseInfoSysncConfigOutput ent : entList) {
            try {
                CommonDataService baseService = NamedServiceProvider.getServiceByName(context, ent.getFromDb().replace("_", "-"), CommonDataService.class);
                if (ent.getLastUpdateTime() == null) {
                    ent.setLastUpdateTime(XDateUtils.addDays(nowDate, -1)); //默认同步一天的数据
                }
                String sql = (ent.getIsFullSync()!=null && ent.getIsFullSync()==1)?
                        String.format("select * from %s ",ent.getTableName()):
                        String.format("select * from %s where %s > '%s' or %s > '%s'" , ent.getTableName(), ent.getCreateTimeField(), XDateUtils.getString(ent.getLastUpdateTime()), ent.getModifyTimeField(), XDateUtils.getString(ent.getLastUpdateTime()));

                CommonDataService dataService = NamedServiceProvider.getServiceByName(context, ent.getTargetDb().replace("_", "-"), CommonDataService.class);
                SelectCommonInput selectCommonInput = new SelectCommonInput();
                selectCommonInput.setCmd(sql);
                selectCommonInput.setTraceId(context.getTraceId());
                XSingleResult<SelectCommonOutput> serviceResult = baseService.selectCommon(context, selectCommonInput);
                serviceResult.throwIfFail();
                List<LinkedHashMap<String, Object>> resultList = serviceResult.getResult().getResultList();
                if (CollectionUtils.isEmpty(resultList)) {
                    continue;
                }

                GetColumnListInput columnInput = new GetColumnListInput();
                columnInput.setTableName(ent.getTableName());
                columnInput.setModelName(ent.getTargetDb());
                XListResult<GetColumnListOutput> columnResult = dataService.getColumnList(context, columnInput);
                columnResult.throwIfFail();
                List<GetColumnListOutput> columnList = columnResult.getResult();

                if (CollectionUtils.isEmpty(columnList)) {
                    context.getLogger().error("目标库表不存在：" + ent.getTableName());
                    continue;
                }

                XTransactionHelper.begin(context, () -> {
                    //从目标库删除
                    String deleteSql = null;
                    if(ent.getIsFullSync()!=null && ent.getIsFullSync()==1){
                        deleteSql = String.format("truncate %s ", ent.getTableName());
                    }else{
                        String idStr = resultList.stream().map(x -> "'" + String.valueOf(x.get("id")) + "'").collect(Collectors.joining(","));
                        deleteSql = String.format("delete from %s where id in (%s) ", ent.getTableName(), idStr);
                    }
                    SelectCommonInput deleteCommonInput = new SelectCommonInput();
                    deleteCommonInput.setCmd(deleteSql);
                    deleteCommonInput.setTraceId(context.getTraceId());
                    XSingleResult<SelectCommonOutput> deleteResult = dataService.selectCommon(context, deleteCommonInput);
                    deleteResult.throwIfFail();

                    //插入目标库
                    List<String> columnNameList = columnList.stream().map(x -> x.getColumnName()).collect(Collectors.toList());
                    StringBuilder sb = new StringBuilder();
                    String columnNameStr = columnNameList.stream().collect(Collectors.joining(","));
                    sb.append("INSERT INTO ").append(ent.getTableName()).append(" (").append(columnNameStr).append(") values ");
                    for (LinkedHashMap<String, Object> dataMap : resultList) {
                        sb.append("(").append(assemblySql(columnNameList, dataMap)).append("),");
                    }
                    sb.deleteCharAt(sb.length() - 1);

                    SelectCommonInput insertCommonInput = new SelectCommonInput();
                    insertCommonInput.setCmd(sb.toString());
                    insertCommonInput.setTraceId(context.getTraceId());
                    XSingleResult<SelectCommonOutput> insertResult = dataService.selectCommon(context, insertCommonInput);
                    insertResult.throwIfFail();

                    //更新最后更新时间
                    ent.setLastUpdateTime(nowDate);
                    ent.setIsFullSync(0); //每次都需手动打开
                    service.updateSyncConfig(context, ent);
                    return XServiceResult.OK;
                });
            }catch (Exception e){
                context.getLogger().error("同步表"+ent.getTableName()+"数据至库"+ent.getTargetDb()+"时发生异常",e);
            }
        }
        return XServiceResult.OK;
    }

    private StringBuilder assemblySql(List<String> columnNameList, HashMap<String, Object> fields) {
        StringBuilder sb = new StringBuilder();
        for (String columnName : columnNameList) {
            boolean isExists = false;
            Iterator<Map.Entry<String, Object>> iterator = fields.entrySet().iterator();
            while (iterator.hasNext()) {
                Map.Entry<String, Object> next = iterator.next();
                String key = next.getKey();
                if (!columnName.equals(key)) {
                    continue;
                }
                Object value = next.getValue();
                if(String.valueOf(value).contains("'")){
                    sb.append("'").append(String.valueOf(value).replace("'","\"")).append("',");
                }else {
                    sb.append("'").append(value).append("',");
                }
                isExists = true;
                break;
            }
            if (!isExists) {
                sb.append("null").append(",");
            }
        }
        sb.deleteCharAt(sb.length() - 1);
        return sb;
    }
}
